@@ -52,7 +52,8 @@ module Cmd_arg = struct
     | Terminal s -> Memo.return s
     | Expandable (sw, _) ->
       let+ path, _ =
-        Action_builder.run (Target.expand_path_from_root root sctx sw) Eager
+        Target.expand_path_from_root root sctx sw
+        |> Action_builder.evaluate_and_collect_facts
       in
       let context = Dune_rules.Super_context.context sctx in
       (* TODO Why are we stringifying this path? *)
@@ -81,7 +82,8 @@ module Command_to_exec = struct
       let path = Path.to_string path in
       let env = Env.to_unix env |> Spawn.Env.of_list in
       let argv = path :: args in
-      Spawn.spawn ~prog:path ~env ~argv ()
+      let cwd = Spawn.Working_dir.Path Fpath.initial_cwd in
+      Spawn.spawn ~prog:path ~env ~cwd ~argv ()
     in
     Pid.of_int pid
   ;;
@@ -183,10 +185,11 @@ let not_found ~dir ~prog =
        matching executable, they would be located in a subdirectory of
        [dir], so it's unclear if that's what the user wanted. *)
     let+ candidates =
-      Build_system.files_of ~dir:(Path.build dir)
-      >>| Path.Set.to_list
-      >>| List.filter ~f:(fun p -> Path.extension p = ".exe")
-      >>| List.map ~f:(fun p -> "./" ^ Path.basename p)
+      let+ filename_set = Build_system.files_of ~dir:(Path.build dir) in
+      Filename_set.filenames filename_set
+      |> Filename.Set.to_list
+      |> List.filter ~f:(fun filename -> Filename.extension filename = ".exe")
+      |> List.map ~f:(fun filename -> "./" ^ filename)
     in
     User_message.did_you_mean prog ~candidates
   in
@@ -262,7 +265,7 @@ module Exec_context = struct
       let context = Dune_rules.Super_context.context sctx in
       Path.Build.relative (Context.build_dir context) (Common.prefix_target common "")
     in
-    let env = Memo.map sctx ~f:Super_context.context_env in
+    let env = Memo.bind sctx ~f:Super_context.context_env in
     let get_path_and_build_if_necessary ~prog =
       let* sctx = sctx
       and+ dir = dir in
