@@ -222,13 +222,15 @@ let exec_run_dynamic_client ~display ~ectx ~eenv prog args =
       match ectx.targets with
       | None -> String.Set.empty
       | Some targets ->
-        if not (Path.Build.Set.is_empty targets.dirs)
+        if not (Filename.Set.is_empty targets.dirs)
         then
           User_error.raise
             ~loc:ectx.rule_loc
             [ Pp.text "Directory targets are not compatible with dynamic actions" ];
-        Path.Build.Set.to_list_map targets.files ~f:(fun target ->
-          Path.reach (Path.build target) ~from:eenv.working_dir)
+        Filename.Set.to_list_map targets.files ~f:(fun target ->
+          Path.Build.relative targets.root target
+          |> Path.build
+          |> Path.reach ~from:eenv.working_dir)
         |> String.Set.of_list
     in
     { DAP.Run_arguments.prepared_dependencies = eenv.prepared_dependencies; targets }
@@ -443,7 +445,7 @@ let rec exec t ~display ~ectx ~eenv : done_or_more_deps Produce.t =
     let remove_intermediate_file () =
       if optional
       then (
-        try Path.unlink (Path.build file2) with
+        try Path.unlink_exn (Path.build file2) with
         | Unix.Unix_error (ENOENT, _, _) -> ())
     in
     if diff_eq_files diff
