@@ -23,7 +23,7 @@ val create
   :  super_context:Super_context.t
   -> scope:Scope.t
   -> obj_dir:Path.Build.t Obj_dir.t
-  -> modules:Modules.t
+  -> modules:Modules.With_vlib.t
   -> flags:Ocaml_flags.t
   -> requires_compile:Lib.t list Resolve.Memo.t
   -> requires_link:Lib.t list Resolve.t Memo.Lazy.t
@@ -32,9 +32,9 @@ val create
   -> ?stdlib:Ocaml_stdlib.t
   -> js_of_ocaml:Js_of_ocaml.In_context.t option
   -> package:Package.t option
-  -> ?public_lib_name:Lib_name.t
+  -> melange_package_name:Lib_name.t option
   -> ?vimpl:Vimpl.t
-  -> ?modes:Dune_file.Mode_conf.Set.Details.t Lib_mode.Map.t
+  -> ?modes:Mode_conf.Set.Details.t Lib_mode.Map.t
   -> ?bin_annot:bool
   -> ?loc:Loc.t
   -> unit
@@ -52,7 +52,7 @@ val scope : t -> Scope.t
 val dir : t -> Path.Build.t
 
 val obj_dir : t -> Path.Build.t Obj_dir.t
-val modules : t -> Modules.t
+val modules : t -> Modules.With_vlib.t
 val flags : t -> Ocaml_flags.t
 val requires_link : t -> Lib.t list Resolve.Memo.t
 val requires_compile : t -> Lib.t list Resolve.Memo.t
@@ -65,7 +65,7 @@ val sandbox : t -> Sandbox_config.t
 val set_sandbox : t -> Sandbox_config.t -> t
 val package : t -> Package.t option
 val vimpl : t -> Vimpl.t option
-val public_lib_name : t -> Lib_name.t option
+val melange_package_name : t -> Lib_name.t option
 val modes : t -> Lib_mode.Map.Set.t
 val for_wrapped_compat : t -> t
 val for_root_module : t -> Module.t -> t
