@@ -61,7 +61,7 @@ module Linkage = struct
     (ocaml : Ocaml_toolchain.t)
     ~dynamically_linked_foreign_archives
     ~loc
-    (m : Dune_file.Executables.Link_mode.t)
+    (m : Executables.Link_mode.t)
     =
     match m with
     | Other { mode = Byte; kind = Js } -> js
@@ -87,7 +87,7 @@ module Linkage = struct
       in
       let ext =
         let lib_config = ocaml.lib_config in
-        Dune_file.Executables.Link_mode.extension
+        Executables.Link_mode.extension
           m
           ~loc
           ~ext_obj:lib_config.ext_obj
@@ -278,13 +278,13 @@ let link_many
     Memo.parallel_map programs ~f:(fun { Program.name; main_module_name; loc } ->
       let top_sorted_modules =
         let main =
-          match Modules.find modules main_module_name with
+          match Modules.With_vlib.find modules main_module_name with
           | Some m -> m
           | None ->
             Code_error.raise
               "link_many: unable to find module"
               [ "main_module_name", Module_name.to_dyn main_module_name
-              ; "modules", Modules.to_dyn modules
+              ; "modules", Modules.With_vlib.to_dyn modules
               ]
         in
         Dep_graph.top_closed_implementations
@@ -363,6 +363,10 @@ let build_and_link_many
   =
   let open Memo.O in
   let* () = Module_compilation.build_all cctx in
+  let* () =
+    Memo.when_ (Compilation_context.bin_annot cctx) (fun () ->
+      Ocaml_index.cctx_rules cctx)
+  in
   link_many
     ?link_args
     ?o_files
