@@ -107,7 +107,7 @@ let gen_rules_output
      in
      let open Action_builder.With_targets.O in
      Action_builder.with_no_targets extra_deps
-     >>> Preprocessing.action_for_pp_with_target
+     >>> Pp_spec_rules.action_for_pp_with_target
            ~sandbox:Sandbox_config.default
            ~loc
            ~expander
@@ -122,11 +122,9 @@ let gen_rules_output
   let* () =
     Memo.Option.iter source_dir ~f:(fun source_dir ->
       Source_tree.Dir.filenames source_dir
-      |> Memo.parallel_iter_set
-           (module Filename.Set)
-           ~f:(fun file ->
-             Path.Source.relative (Source_tree.Dir.path source_dir) file
-             |> setup_formatting))
+      |> Filename.Set.to_seq
+      |> Memo.parallel_iter_seq ~f:(fun file ->
+        Path.Source.relative (Source_tree.Dir.path source_dir) file |> setup_formatting))
   and* () =
     match Format_config.includes config Dune with
     | false -> Memo.return ()
