@@ -111,7 +111,11 @@ end = struct
       let+ () = Toplevel.Stanza.setup ~sctx ~dir ~toplevel in
       empty_none
     | Library.T lib ->
-      let* enabled_if = Lib.DB.available (Scope.libs scope) (Library.best_name lib) in
+      let* enabled_if =
+        Lib.DB.available_by_lib_id
+          (Scope.libs scope)
+          (Local (Library.to_lib_id ~src_dir lib))
+      in
       if_available_buildable
         ~loc:lib.buildable.loc
         (fun () -> Lib_rules.rules lib ~sctx ~dir ~scope ~dir_contents ~expander)
@@ -132,7 +136,7 @@ end = struct
         { (with_cctx_merlin ~loc:exes.buildable.loc cctx_merlin) with
           js =
             Some
-              (List.map exes.names ~f:(fun (_, exe) ->
+              (List.map (Nonempty_list.to_list exes.names) ~f:(fun (_, exe) ->
                  Path.Build.relative dir (exe ^ Js_of_ocaml.Ext.exe)))
         })
     | Alias_conf.T alias ->
@@ -207,9 +211,9 @@ let define_all_alias ~dir ~project ~js_targets =
 
 let gen_rules_for_stanzas sctx dir_contents cctxs expander dune_file ~dir:ctx_dir =
   let src_dir = Dune_file.dir dune_file in
-  let* stanzas = Dune_file.stanzas dune_file in
+  let* stanzas = Dune_file.stanzas dune_file
+  and* scope = Scope.DB.find_by_dir ctx_dir in
   let* { For_stanza.merlin = merlins; cctx = cctxs; js = js_targets; source_dirs } =
-    let* scope = Scope.DB.find_by_dir ctx_dir in
     For_stanza.of_stanzas
       stanzas
       ~cctxs
@@ -236,26 +240,27 @@ let gen_rules_for_stanzas sctx dir_contents cctxs expander dune_file ~dir:ctx_di
          | false -> Memo.return ()
          | true ->
            let* ml_sources = Dir_contents.ocaml dir_contents in
-           (match
-              let base_path =
-                match Ml_sources.include_subdirs ml_sources with
-                | Include Unqualified | No -> []
-                | Include Qualified ->
-                  Path.Local.descendant
-                    (Path.Build.local ctx_dir)
-                    ~of_:(Path.Build.local (Dir_contents.dir dir_contents))
-                  |> Option.value_exn
-                  |> Path.Local.explode
-                  |> List.map ~f:Module_name.of_string
-              in
-              Menhir_rules.module_names m
-              |> List.find_map ~f:(fun name ->
-                let open Option.O in
-                let path = base_path @ [ name ] in
-                let* origin = Ml_sources.find_origin ml_sources path in
-                List.find_map cctxs ~f:(fun (loc, cctx) ->
-                  Option.some_if (Loc.equal loc (Ml_sources.Origin.loc origin)) cctx))
-            with
+           let base_path =
+             match Ml_sources.include_subdirs ml_sources with
+             | Include Unqualified | No -> []
+             | Include Qualified ->
+               Path.Local.descendant
+                 (Path.Build.local ctx_dir)
+                 ~of_:(Path.Build.local (Dir_contents.dir dir_contents))
+               |> Option.value_exn
+               |> Path.Local.explode
+               |> List.map ~f:Module_name.of_string
+           in
+           Menhir_rules.module_names m
+           |> Memo.List.find_map ~f:(fun name ->
+             let path = base_path @ [ name ] in
+             Ml_sources.find_origin ml_sources ~libs:(Scope.libs scope) path
+             >>| function
+             | None -> None
+             | Some origin ->
+               List.find_map cctxs ~f:(fun (loc, cctx) ->
+                 Option.some_if (Loc.equal loc (Ml_sources.Origin.loc origin)) cctx))
+           >>= (function
             | Some cctx -> Menhir_rules.gen_rules cctx m ~dir:ctx_dir
             | None ->
               (* This happens often when passing a [-p ...] option that hides a
@@ -348,6 +353,7 @@ let gen_project_rules =
     let+ () = Install_rules.gen_project_rules sctx project
     and+ () = Odoc.gen_project_rules sctx project
     and+ () = Odoc_new.gen_project_rules sctx project
+    and+ () = Ocaml_index.project_rule sctx project
     and+ () =
       let version = 2, 8 in
       match Dune_project.allow_approximate_merlin project with
@@ -606,7 +612,7 @@ let gen_rules ctx sctx ~dir components : Gen_rules.result Memo.t =
        | _ -> Subdir_set.empty)
       (fun () ->
         let* sctx = sctx in
-        Preprocessing.gen_rules sctx rest)
+        Pp_spec_rules.gen_rules sctx rest)
   | [ ".dune" ] ->
     has_rules
       ~dir
