@@ -64,8 +64,10 @@ let check_runtime_deps_relative_path local_path ~loc ~lib_info =
       ]
       ~hints:
         [ Pp.textf
-            "Move the dependency to a descendant of the folder where the library is \
-             defined"
+            "Move the offending dependency somewhere inside `%s'."
+            (Path.build lib_src_dir
+             |> Path.drop_optional_build_context_src_exn
+             |> Path.Source.to_string)
         ]
   | Some _ -> ()
 ;;
@@ -80,15 +82,20 @@ end = struct
     | Normal | Ppx_deriver _ -> Memo.return []
     | Ppx_rewriter _ ->
       Library.best_name lib
-      |> Preprocessing.ppx_exe ctx ~scope
+      |> Ppx_driver.ppx_exe ctx ~scope
       |> Resolve.Memo.read_memo
       >>| List.singleton
   ;;
 
-  let lib_files ~dir_contents ~dir ~lib_config lib =
+  let lib_files ~scope ~dir_contents ~dir ~lib_config lib =
     let+ modules =
-      let+ ml_sources = Dir_contents.ocaml dir_contents in
-      Some (Ml_sources.modules ml_sources ~for_:(Library (Lib_info.name lib)))
+      let* ml_sources = Dir_contents.ocaml dir_contents in
+      Ml_sources.modules
+        ml_sources
+        ~libs:(Scope.libs scope)
+        ~for_:(Library (Lib_info.lib_id lib |> Lib_id.to_local_exn))
+      >>| Modules.With_vlib.modules
+      >>| Option.some
     and+ foreign_archives =
       match Lib_info.virtual_ lib with
       | None -> Memo.return (Mode.Map.Multi.to_flat_list @@ Lib_info.foreign_archives lib)
@@ -181,9 +188,12 @@ end = struct
     let lib_name = Library.best_name lib in
     let* installable_modules =
       let+ modules =
-        Dir_contents.ocaml dir_contents >>| Ml_sources.modules ~for_:(Library lib_name)
+        Dir_contents.ocaml dir_contents
+        >>= Ml_sources.modules
+              ~libs:(Scope.libs scope)
+              ~for_:(Library (Lib_info.lib_id info |> Lib_id.to_local_exn))
       and+ impl = Virtual_rules.impl sctx ~lib ~scope in
-      Vimpl.impl_modules impl modules |> Modules.split_by_lib
+      Vimpl.impl_modules impl modules |> Modules.With_vlib.split_by_lib
     in
     let lib_src_dir = Lib_info.src_dir info in
     let sources =
@@ -305,7 +315,7 @@ end = struct
           if Module.kind m = Virtual then [] else common m |> set_dir m)
       in
       modules_vlib @ modules_impl
-    and+ lib_files = lib_files ~dir ~dir_contents ~lib_config info
+    and+ lib_files = lib_files ~scope ~dir ~dir_contents ~lib_config info
     and+ execs = lib_ppxs ctx ~scope ~lib
     and+ dll_files =
       dll_files ~modes:ocaml ~dynlink:lib.dynlink ~ctx info
@@ -338,7 +348,15 @@ end = struct
         if enabled_if
         then
           if lib.optional
-          then Lib.DB.available (Scope.libs scope) (Library.best_name lib)
+          then (
+            let src_dir =
+              Expander.dir expander
+              |> Path.build
+              |> Path.drop_optional_build_context_src_exn
+            in
+            Lib.DB.available_by_lib_id
+              (Scope.libs scope)
+              (Local (Library.to_lib_id ~src_dir lib)))
           else Memo.return true
         else Memo.return false
       | Documentation.T _ -> Memo.return true
@@ -365,13 +383,12 @@ end = struct
                  |> Resolve.Memo.read_memo
                  >>| Preprocess.Per_module.pps
                in
-               let merlin_ident =
-                 Merlin_ident.for_exes ~names:(List.map ~f:snd exes.names)
-               in
+               let names = Nonempty_list.to_list exes.names in
+               let merlin_ident = Merlin_ident.for_exes ~names:(List.map ~f:snd names) in
                Lib.DB.resolve_user_written_deps
                  (Scope.libs scope)
                  ~forbidden_libraries:[]
-                 (`Exe exes.names)
+                 (`Exe names)
                  exes.buildable.libraries
                  ~pps
                  ~dune_version
@@ -653,7 +670,12 @@ end = struct
             |> Foreign.Sources.object_files ~dir ~ext_obj
             |> List.map ~f:Path.build
           and* modules =
-            Dir_contents.ocaml dir_contents >>| Ml_sources.modules ~for_:(Library name)
+            let* libs = Scope.DB.find_by_dir dir >>| Scope.libs in
+            Dir_contents.ocaml dir_contents
+            >>= Ml_sources.modules
+                  ~libs
+                  ~for_:(Library (Lib_info.lib_id info |> Lib_id.to_local_exn))
+            >>| Modules.With_vlib.modules
           and* melange_runtime_deps = file_deps (Lib_info.melange_runtime_deps info)
           and* public_headers = file_deps (Lib_info.public_headers info) in
           let+ dune_lib =
@@ -715,7 +737,10 @@ end = struct
                 ~then_:(Action_builder.return Dune_package.Or_meta.Use_meta)
                 ~else_:(make_dune_package sctx lib_entries pkg)
          in
-         Format.asprintf "%a" (Dune_package.Or_meta.pp ~dune_version) pkg)
+         Format.asprintf
+           "%a"
+           (Dune_package.Or_meta.pp ~dune_version ~encoding:Relative)
+           pkg)
     in
     let* () =
       let deprecated_dune_packages =
@@ -768,7 +793,7 @@ end = struct
             (Package_paths.deprecated_dune_package_file ctx pkg dune_pkg.name)
             (Format.asprintf
                "%a"
-               (Dune_package.Or_meta.pp ~dune_version)
+               (Dune_package.Or_meta.pp ~dune_version ~encoding:Relative)
                (Dune_package dune_pkg))
         in
         Super_context.add_rule sctx ~dir:ctx.build_dir ~loc action_with_targets)
@@ -848,7 +873,8 @@ end = struct
     in
     let deprecated_packages = Package.Name.Map.of_list_multi deprecated_packages in
     Package.deprecated_package_names pkg
-    |> Package_map_traversals.parallel_iter ~f:(fun name loc ->
+    |> Dune_lang.Package_name.Map.to_seq
+    |> Memo.parallel_iter_seq ~f:(fun (name, loc) ->
       let meta = Package_paths.deprecated_meta_file ctx pkg name in
       Super_context.add_rule
         sctx
@@ -872,7 +898,8 @@ end = struct
 
   let meta_and_dune_package_rules sctx project =
     Dune_project.packages project
-    |> Package_map_traversals.parallel_iter ~f:(fun _name (pkg : Package.t) ->
+    |> Dune_lang.Package_name.Map.to_seq
+    |> Memo.parallel_iter_seq ~f:(fun (_name, (pkg : Package.t)) ->
       gen_dune_package sctx pkg >>> gen_meta_file sctx pkg)
   ;;
 end
@@ -1341,7 +1368,8 @@ let stanzas_to_entries = Stanzas_to_entries.stanzas_to_entries
 let gen_project_rules sctx project =
   let* () = meta_and_dune_package_rules sctx project in
   Dune_project.packages project
-  |> Package_map_traversals.parallel_iter ~f:(fun _name package ->
+  |> Dune_lang.Package_name.Map.to_seq
+  |> Memo.parallel_iter_seq ~f:(fun (_name, package) ->
     let* () = gen_package_install_file_rules sctx package in
     gen_install_alias sctx package)
 ;;
