@@ -300,6 +300,7 @@ end
 type 'path t =
   { loc : Loc.t
   ; name : Lib_name.t
+  ; lib_id : Lib_id.t
   ; kind : Lib_kind.t
   ; status : Status.t
   ; src_dir : 'path
@@ -329,7 +330,7 @@ type 'path t =
   ; wrapped : Wrapped.t Inherited.t option
   ; main_module_name : Main_module_name.t
   ; modes : Lib_mode.Map.Set.t
-  ; modules : Modules.t option Source.t
+  ; modules : Modules.With_vlib.t option Source.t
   ; special_builtin_support : (Loc.t * Special_builtin_support.t) option
   ; exit_module : Module_name.t option
   ; instrumentation_backend : (Loc.t * Lib_name.t) option
@@ -338,6 +339,7 @@ type 'path t =
   }
 
 let name t = t.name
+let lib_id t = t.lib_id
 let version t = t.version
 let dune_version t = t.dune_version
 let loc t = t.loc
@@ -379,7 +381,8 @@ let eval_native_archives_exn (type path) (t : path t) ~modules =
   match t.native_archives, modules with
   | Files f, _ -> f
   | Needs_module_info _, None -> Code_error.raise "missing module information" []
-  | Needs_module_info f, Some modules -> if Modules.has_impl modules then [ f ] else []
+  | Needs_module_info f, Some modules ->
+    if Modules.With_vlib.has_impl modules then [ f ] else []
 ;;
 
 let user_written_deps t =
@@ -391,6 +394,7 @@ let create
   ~loc
   ~path_kind
   ~name
+  ~lib_id
   ~kind
   ~status
   ~src_dir
@@ -428,6 +432,7 @@ let create
   =
   { loc
   ; name
+  ; lib_id
   ; kind
   ; status
   ; src_dir
@@ -520,6 +525,7 @@ let to_dyn
   { loc
   ; path_kind = _
   ; name
+  ; lib_id
   ; kind
   ; status
   ; src_dir
@@ -561,6 +567,7 @@ let to_dyn
   record
     [ "loc", Loc.to_dyn_hum loc
     ; "name", Lib_name.to_dyn name
+    ; "lib_id", Lib_id.to_dyn lib_id
     ; "kind", Lib_kind.to_dyn kind
     ; "status", Status.to_dyn status
     ; "src_dir", path src_dir
@@ -591,7 +598,7 @@ let to_dyn
     ; "wrapped", option (Inherited.to_dyn Wrapped.to_dyn) wrapped
     ; "main_module_name", Main_module_name.to_dyn main_module_name
     ; "modes", Lib_mode.Map.Set.to_dyn modes
-    ; "modules", Source.to_dyn (Dyn.option Modules.to_dyn) modules
+    ; "modules", Source.to_dyn (Dyn.option Modules.With_vlib.to_dyn) modules
     ; ( "special_builtin_support"
       , option (snd Special_builtin_support.to_dyn) special_builtin_support )
     ; "exit_module", option Module_name.to_dyn exit_module
