@@ -120,14 +120,14 @@ let build_lib
 
 let gen_wrapped_compat_modules (lib : Library.t) cctx =
   let modules = Compilation_context.modules cctx in
-  let wrapped_compat = Modules.wrapped_compat modules in
+  let wrapped_compat = Modules.With_vlib.wrapped_compat modules in
   let transition_message =
     lazy
-      (match Modules.wrapped modules with
+      (match Modules.With_vlib.wrapped modules with
        | Simple _ -> assert false
        | Yes_with_transition r -> r)
   in
-  Module_name.Map_traversals.parallel_iter wrapped_compat ~f:(fun name m ->
+  Memo.parallel_iter_seq (Module_name.Map.to_seq wrapped_compat) ~f:(fun (name, m) ->
     let main_module_name =
       match Library.main_module_name lib with
       | This (Some mmn) -> Module_name.to_string mmn
@@ -439,7 +439,7 @@ let setup_build_archives (lib : Library.t) ~top_sorted_modules ~cctx ~expander ~
   let { Lib_config.ext_obj; natdynlink_supported; _ } = ocaml.lib_config in
   let open Memo.O in
   let* () =
-    Modules.exit_module modules
+    Modules.With_vlib.exit_module modules
     |> Memo.Option.iter ~f:(fun m ->
       (* These files needs to be alongside stdlib.cma as the compiler
          implicitly adds this module. *)
@@ -528,14 +528,16 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope ~compile_
   in
   let package = Library.package lib in
   let js_of_ocaml = Js_of_ocaml.In_context.make ~dir lib.buildable.js_of_ocaml in
-  (* XXX(anmonteiro): `public_lib_name` is used to derive Melange's
+  (* XXX(anmonteiro): `melange_package_name` is used to derive Melange's
      `--bs-package-name` argument. We only use the library name for public
-     libraries because we need melange to preserve relative paths for private
-     libs (i.e. not pass the `--bs-package-name` arg). *)
-  let public_lib_name =
+     libraries / private libraries with `(package ..)` because we need Melange
+     to preserve relative paths for private libs (i.e. not pass the
+     `--bs-package-name` arg). *)
+  let melange_package_name =
     match lib.visibility with
     | Public p -> Some (Public_lib.name p)
-    | Private _ -> None
+    | Private (Some pkg) -> Some (Lib_name.mangled (Package.name pkg) (snd lib.name))
+    | Private None -> None
   in
   Compilation_context.create
     ()
@@ -552,7 +554,7 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope ~compile_
     ?stdlib:lib.stdlib
     ~package
     ?vimpl
-    ?public_lib_name
+    ~melange_package_name
     ~modes
 ;;
 
@@ -577,9 +579,10 @@ let library_rules
   let scope = Compilation_context.scope cctx in
   let* requires_compile = Compilation_context.requires_compile cctx in
   let ocaml = Compilation_context.ocaml cctx in
+  let* requires_link = Compilation_context.requires_link cctx in
   let stdlib_dir = ocaml.lib_config.stdlib_dir in
   let top_sorted_modules =
-    let impl_only = Modules.impl_only modules in
+    let impl_only = Modules.With_vlib.impl_only modules in
     Dep_graph.top_closed_implementations
       (Compilation_context.dep_graphs cctx).impl
       impl_only
@@ -605,6 +608,9 @@ let library_rules
     info
   in
   let+ () =
+    Memo.when_ (Compilation_context.bin_annot cctx) (fun () ->
+      Ocaml_index.cctx_rules cctx)
+  and+ () =
     Memo.when_
       (not (Library.is_virtual lib))
       (fun () -> setup_build_archives lib ~lib_info ~top_sorted_modules ~cctx ~expander)
@@ -629,7 +635,7 @@ let library_rules
   in
   ( cctx
   , Merlin.make
-      ~requires:requires_compile
+      ~requires:requires_link
       ~stdlib_dir
       ~flags
       ~modules
@@ -645,16 +651,20 @@ let library_rules
 let rules (lib : Library.t) ~sctx ~dir_contents ~dir ~expander ~scope =
   let buildable = lib.buildable in
   let* local_lib, compile_info =
+    let src_dir = Path.Build.drop_build_context_exn dir in
     Lib.DB.get_compile_info
       (Scope.libs scope)
-      (Library.best_name lib)
+      (Local (Library.to_lib_id ~src_dir lib))
       ~allow_overlaps:buildable.allow_overlapping_dependencies
   in
   let local_lib = Lib.Local.of_lib_exn local_lib in
   let f () =
     let* source_modules =
       Dir_contents.ocaml dir_contents
-      >>| Ml_sources.modules ~for_:(Library (Library.best_name lib))
+      >>= Ml_sources.modules
+            ~libs:(Scope.libs scope)
+            ~for_:
+              (Library (Lib_info.lib_id (Lib.Local.info local_lib) |> Lib_id.to_local_exn))
     in
     let* cctx = cctx lib ~sctx ~source_modules ~dir ~scope ~expander ~compile_info in
     let* () =
