@@ -1,17 +1,24 @@
 open Import
 open Memo.O
 
-let output_of_lib ~target_dir lib =
-  let info = Lib.info lib in
-  match Lib_info.status info with
-  | Private _ -> `Private_library_or_emit target_dir
-  | Installed | Installed_private | Public _ ->
-    let lib_name = Lib_info.name info in
-    let src_dir = Lib_info.src_dir info in
+let output_of_lib =
+  let public_lib ~info ~target_dir lib_name =
     `Public_library
-      ( src_dir
+      ( Lib_info.src_dir info
       , Path.Build.L.relative target_dir [ "node_modules"; Lib_name.to_string lib_name ]
       )
+  in
+  fun ~target_dir lib ->
+    let info = Lib.info lib in
+    match Lib_info.status info with
+    | Private (_, None) -> `Private_library_or_emit target_dir
+    | Private (_, Some pkg) ->
+      public_lib
+        ~info
+        ~target_dir
+        (Lib_name.mangled (Package.name pkg) (Lib_name.to_local_exn (Lib.name lib)))
+    | Installed | Installed_private | Public _ ->
+      public_lib ~info ~target_dir (Lib_info.name info)
 ;;
 
 let lib_output_path ~output_dir ~lib_dir src =
@@ -50,14 +57,13 @@ let modules_in_obj_dir ~sctx ~scope ~preprocess modules =
          preprocess
          ~instrumentation_backend:(Lib.DB.instrumentation_backend (Scope.libs scope)))
   in
-  let pped_map = Staged.unstage (Preprocessing.pped_modules_map preprocess version) in
+  let pped_map = Staged.unstage (Pp_spec.pped_modules_map preprocess version) in
   Modules.map_user_written modules ~f:(fun m -> Memo.return @@ pped_map m)
 ;;
 
 let impl_only_modules_defined_in_this_lib ~sctx ~scope lib =
-  let* modules = Dir_contents.modules_of_lib sctx lib in
-  match modules with
-  | None ->
+  match Lib_info.modules (Lib.info lib) with
+  | External None ->
     User_error.raise
       [ Pp.textf
           "The library %s was not compiled with Dune or it was compiled with Dune but \
@@ -65,11 +71,18 @@ let impl_only_modules_defined_in_this_lib ~sctx ~scope lib =
            melange support"
           (Lib.name lib |> Lib_name.to_string)
       ]
-  | Some modules ->
-    let info = Lib.info lib in
+  | External (Some modules) ->
+    Memo.return
+      ( modules
+      , (Modules.With_vlib.split_by_lib modules).impl
+        |> List.filter ~f:(Module.has ~ml_kind:Impl) )
+  | Local ->
+    let lib = Lib.Local.of_lib_exn lib in
+    let info = Lib.Local.info lib in
     let+ modules =
+      let* modules = Dir_contents.modules_of_local_lib sctx lib in
       let preprocess = Lib_info.preprocess info in
-      modules_in_obj_dir ~sctx ~scope ~preprocess modules
+      modules_in_obj_dir ~sctx ~scope ~preprocess modules >>| Modules.With_vlib.modules
     in
     let () =
       let modes = Lib_info.modes info in
@@ -88,8 +101,8 @@ let impl_only_modules_defined_in_this_lib ~sctx ~scope lib =
       | true -> ()
     in
     ( modules
-    , (* for a virtual library, this will return all modules *)
-      (Modules.split_by_lib modules).impl |> List.filter ~f:(Module.has ~ml_kind:Impl) )
+    , (Modules.With_vlib.split_by_lib modules).impl
+      |> List.filter ~f:(Module.has ~ml_kind:Impl) )
 ;;
 
 let cmj_glob = Glob.of_string_exn Loc.none "*.cmj"
@@ -109,11 +122,12 @@ let cmj_includes ~(requires_link : Lib.t list Resolve.t) ~scope =
   let+ requires_link = requires_link in
   let deps = List.map requires_link ~f:deps_of_lib |> Dep.Set.of_list in
   Command.Args.S
-    [ Lib_flags.L.include_flags ~project requires_link Melange; Hidden_deps deps ]
+    [ Lib_flags.L.melange_emission_include_flags ~project requires_link
+    ; Hidden_deps deps
+    ]
 ;;
 
 let compile_info ~scope (mel : Melange_stanzas.Emit.t) =
-  let open Memo.O in
   let dune_version = Scope.project scope |> Dune_project.dune_version in
   let+ pps =
     Resolve.Memo.read_memo
@@ -143,7 +157,9 @@ let compile_info ~scope (mel : Melange_stanzas.Emit.t) =
 
 let js_targets_of_modules modules ~module_systems ~output =
   List.map module_systems ~f:(fun (_, js_ext) ->
-    Modules.fold_no_vlib modules ~init:Path.Set.empty ~f:(fun m acc ->
+    modules
+    |> Modules.With_vlib.drop_vlib
+    |> Modules.fold ~init:Path.Set.empty ~f:(fun m acc ->
       if Module.has m ~ml_kind:Impl
       then (
         let target = Path.build @@ make_js_name ~js_ext ~output m in
@@ -154,7 +170,6 @@ let js_targets_of_modules modules ~module_systems ~output =
 
 let js_targets_of_libs ~sctx ~scope ~module_systems ~target_dir libs =
   Resolve.Memo.List.concat_map module_systems ~f:(fun (_, js_ext) ->
-    let open Memo.O in
     let of_lib lib =
       let+ _, modules = impl_only_modules_defined_in_this_lib ~sctx ~scope lib in
       let output = output_of_lib ~target_dir lib in
@@ -184,7 +199,6 @@ let build_js
   ~local_modules_and_obj_dir
   m
   =
-  let open Memo.O in
   let* compiler = Melange_binary.melc sctx ~loc:(Some loc) ~dir in
   Memo.parallel_iter module_systems ~f:(fun (module_system, js_ext) ->
     let build =
@@ -255,32 +269,33 @@ let setup_emit_cmj_rules
   ~dir_contents
   (mel : Melange_stanzas.Emit.t)
   =
-  let open Memo.O in
   let* compile_info = compile_info ~scope mel in
   let ctx = Super_context.context sctx in
   let f () =
-    (* Use "mobjs" rather than "objs" to avoid a potential conflict with a library
-       of the same name *)
     let* modules, obj_dir =
       Dir_contents.ocaml dir_contents
-      >>| Ml_sources.modules_and_obj_dir ~for_:(Melange { target = mel.target })
+      >>= Ml_sources.modules_and_obj_dir
+            ~libs:(Scope.libs scope)
+            ~for_:(Melange { target = mel.target })
     in
     let* () = Check_rules.add_obj_dir sctx ~obj_dir Melange in
     let* modules, pp =
-      Buildable_rules.modules_rules
-        sctx
-        (Melange
-           { preprocess = mel.preprocess
-           ; preprocessor_deps = mel.preprocessor_deps
-           ; (* TODO still needed *)
-             lint = Preprocess.Per_module.default ()
-           ; (* why is this always false? *)
-             empty_module_interface_if_absent = false
-           })
-        expander
-        ~dir
-        scope
-        modules
+      let+ modules, pp =
+        Buildable_rules.modules_rules
+          sctx
+          (Melange
+             { preprocess = mel.preprocess
+             ; preprocessor_deps = mel.preprocessor_deps
+             ; lint = mel.lint
+             ; (* why is this always false? *)
+               empty_module_interface_if_absent = false
+             })
+          expander
+          ~dir
+          scope
+          modules
+      in
+      Modules.With_vlib.modules modules, pp
     in
     let requires_link = Lib.Compile.requires_link compile_info in
     let* flags =
@@ -304,6 +319,7 @@ let setup_emit_cmj_rules
         ~preprocessing:pp
         ~js_of_ocaml:None
         ~opaque:Inherit_from_settings
+        ~melange_package_name:None
         ~package:mel.package
         ~modes:
           { ocaml = { byte = None; native = None }; melange = Some (Requested mel.loc) }
@@ -356,7 +372,6 @@ let setup_emit_cmj_rules
 
 module Runtime_deps = struct
   let targets sctx ~dir ~output ~for_ (mel : Melange_stanzas.Emit.t) =
-    let open Memo.O in
     let raise_external_dep_error src =
       let lib_info =
         match for_ with
@@ -407,7 +422,6 @@ module Runtime_deps = struct
 end
 
 let setup_runtime_assets_rules sctx ~dir ~target_dir ~mode ~output ~for_ mel =
-  let open Memo.O in
   let* copy, non_copy = Runtime_deps.targets sctx ~dir ~output ~for_ mel in
   let deps =
     Action_builder.paths
@@ -422,16 +436,15 @@ let setup_runtime_assets_rules sctx ~dir ~target_dir ~mode ~output ~for_ mel =
 ;;
 
 let modules_for_js_and_obj_dir ~sctx ~dir_contents ~scope (mel : Melange_stanzas.Emit.t) =
-  let open Memo.O in
-  (* Use "mobjs" rather than "objs" to avoid a potential conflict with a library
-     of the same name *)
   let* modules, obj_dir =
     Dir_contents.ocaml dir_contents
-    >>| Ml_sources.modules_and_obj_dir ~for_:(Melange { target = mel.target })
+    >>= Ml_sources.modules_and_obj_dir
+          ~libs:(Scope.libs scope)
+          ~for_:(Melange { target = mel.target })
   in
   let+ modules = modules_in_obj_dir ~sctx ~scope ~preprocess:mel.preprocess modules in
   let modules_for_js =
-    Modules.fold_no_vlib modules ~init:[] ~f:(fun x acc ->
+    Modules.fold modules ~init:[] ~f:(fun x acc ->
       if Module.has x ~ml_kind:Impl then x :: acc else acc)
   in
   modules, modules_for_js, obj_dir
@@ -447,7 +460,6 @@ let setup_entries_js
   ~mode
   (mel : Melange_stanzas.Emit.t)
   =
-  let open Memo.O in
   let* local_modules, modules_for_js, local_obj_dir =
     modules_for_js_and_obj_dir ~sctx ~dir_contents ~scope mel
   in
@@ -462,8 +474,10 @@ let setup_entries_js
   let* () =
     setup_runtime_assets_rules sctx ~dir ~target_dir ~mode ~output ~for_:`Emit mel
   in
+  let local_modules_and_obj_dir =
+    Some (Modules.With_vlib.modules local_modules, local_obj_dir)
+  in
   Memo.parallel_iter modules_for_js ~f:(fun m ->
-    let local_modules_and_obj_dir = Some (local_modules, local_obj_dir) in
     build_js
       ~dir
       ~loc
@@ -485,14 +499,14 @@ let setup_js_rules_libraries =
       let obj_dir = Lib.Local.obj_dir lib in
       modules, obj_dir)
   in
-  let parallel_build_source_modules ~sctx ~scope ~f lib =
+  let parallel_build_source_modules ~sctx ~scope ~f:build_js lib =
     let* local_modules_and_obj_dir, source_modules =
       let+ lib_modules, source_modules =
         impl_only_modules_defined_in_this_lib ~sctx ~scope lib
       in
       local_modules_and_obj_dir ~lib lib_modules, source_modules
     in
-    Memo.parallel_iter source_modules ~f:(f ~local_modules_and_obj_dir)
+    Memo.parallel_iter source_modules ~f:(build_js ~local_modules_and_obj_dir)
   in
   fun ~dir ~scope ~target_dir ~sctx ~requires_link ~mode (mel : Melange_stanzas.Emit.t) ->
     let build_js = build_js ~sctx ~mode ~module_systems:mel.module_systems in
@@ -513,7 +527,6 @@ let setup_js_rules_libraries =
           | Some sub -> if Lib.equal sub lib then acc else sub :: acc)
     in
     Memo.parallel_iter requires_link ~f:(fun lib ->
-      let open Memo.O in
       let lib_compile_info =
         Lib.Compile.for_lib
           ~allow_overlaps:mel.allow_overlapping_dependencies
@@ -601,7 +614,6 @@ let setup_js_rules_libraries_and_entries
   ~target_dir
   mel
   =
-  let open Memo.O in
   let+ () =
     setup_js_rules_libraries ~dir ~scope ~target_dir ~sctx ~requires_link ~mode mel
   and+ () =
@@ -611,7 +623,6 @@ let setup_js_rules_libraries_and_entries
 ;;
 
 let setup_emit_js_rules ~dir_contents ~dir ~scope ~sctx mel =
-  let open Memo.O in
   let target_dir =
     Melange_stanzas.Emit.target_dir ~dir:(Dir_contents.dir dir_contents) mel
   in
@@ -643,7 +654,7 @@ let setup_emit_js_rules ~dir_contents ~dir ~scope ~sctx mel =
        package). When resolution fails, we replace the JS entries with the
        resolution error inside [Action_builder.fail] to give Dune a chance to
        fail if any of the targets end up attached to a package installation. *)
-    let* _local_modules, modules_for_js, _obj_dir =
+    let* _, modules_for_js, _obj_dir =
       modules_for_js_and_obj_dir ~sctx ~dir_contents ~scope mel
     in
     let module_systems = mel.module_systems in
@@ -721,7 +732,7 @@ let gen_emit_rules sctx ~dir ({ stanza_dir; stanza } as for_melange) =
     under_melange_emit_target ~sctx ~dir:stanza_dir
     >>| (function
      | None -> Some (emit_rules sctx for_melange)
-     | Some { stanza_dir = parent_melange_emit_dir; stanza = parent_stanza } ->
+     | Some { stanza_dir = _; stanza = parent_stanza } ->
        let main_message = Pp.text "melange.emit stanzas cannot be nested" in
        let annots =
          let main = User_message.make ~loc:stanza.loc [ main_message ] in
@@ -741,16 +752,7 @@ let gen_emit_rules sctx ~dir ({ stanza_dir; stanza } as for_melange) =
          [ main_message
          ; Pp.enumerate ~f:Loc.pp_file_colon_line [ parent_stanza.loc; stanza.loc ]
          ]
-         ~hints:
-           (let emit_dir = Path.Build.drop_build_context_exn stanza_dir in
-            let parent_melange_emit_dir =
-              Path.Build.drop_build_context_exn parent_melange_emit_dir
-            in
-            [ Pp.textf
-                "Move the melange.emit stanza from %s to at least the level of %s"
-                (Path.Source.to_string emit_dir)
-                (Path.Source.to_string parent_melange_emit_dir)
-            ]))
+         ~hints:[ Pp.text "Move both `melange.emit' stanzas to the same level." ])
 ;;
 
 module Gen_rules = Import.Build_config.Gen_rules
